"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ShipmentExtraReturnServiceTypeLasershipEnum(str, Enum):
    NO_RETURN = 'NO_RETURN'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from shippo import utils


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class TrackingStatusSubstatus:
    r"""A finer-grained classification of the tracking event."""
    code: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('code') }})
    r"""A code that represents the substatus of the shipment. See the <a href=\\"https://docs.goshippo.com/docs/tracking/tracking/#event-definitions\\">Event Definitions</a> for more information."""
    text: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('text') }})
    r"""A human-readable description of the substatus. See the <a href=\\"https://docs.goshippo.com/docs/tracking/tracking/#event-definitions\\">Event Definitions</a> for more information."""
    action_required: bool = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('action_required') }})
    r"""Indicates whether the substatus requires action from the shipper or recipient to complete delivery."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelBetterTrucksEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | better_trucks_same_day | Same Day|
    | better_trucks_next_day | Next Day|
    """
    BETTER_TRUCKS_SAME_DAY = 'better_trucks_same_day'
    BETTER_TRUCKS_NEXT_DAY = 'better_trucks_next_day'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelDHLExpressEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | dhl_express_domestic_express_doc | Domestic Express Doc|
    | dhl_express_economy_select_doc | Economy Select Doc|
    | dhl_express_worldwide_nondoc | Express Worldwide Nondoc|
    | dhl_express_worldwide_doc | Express Worldwide Doc|
    | dhl_express_worldwide | Worldwide|
    | dhl_express_worldwide_eu_doc | Express Worldwide EU Doc|
    | dhl_express_break_bulk_express_doc | Break Bulk Express Doc|
    | dhl_express_express_9_00_nondoc | Express 9:00 NonDoc|
    | dhl_express_economy_select_nondoc | Economy Select NonDoc|
    | dhl_express_break_bulk_economy_doc | Break Bulk Economy Doc|
    | dhl_express_express_9_00_doc | Express 9:00 Doc|
    | dhl_express_express_10_30_doc | Express 10:30 Doc|
    | dhl_express_express_10_30_nondoc | Express 10:30 NonDoc|
    | dhl_express_express_12_00_doc | Express 12:00 Doc|
    | dhl_express_europack_nondoc | Europack NonDoc|
    | dhl_express_express_envelope_doc | Express Envelope Doc|
    | dhl_express_express_12_00_nondoc | Express 12:00 NonDoc|
    | dhl_express_express_12_doc | Domestic Express 12:00|
    | dhl_express_worldwide_b2c_doc | Express Worldwide (B2C) Doc|
    | dhl_express_worldwide_b2c_nondoc | Express Worldwide (B2C) NonDoc|
    | dhl_express_medical_express | Medical Express|
    | dhl_express_express_easy_nondoc | Express Easy NonDoc|
    """
    DHL_EXPRESS_DOMESTIC_EXPRESS_DOC = 'dhl_express_domestic_express_doc'
    DHL_EXPRESS_ECONOMY_SELECT_DOC = 'dhl_express_economy_select_doc'
    DHL_EXPRESS_WORLDWIDE_NONDOC = 'dhl_express_worldwide_nondoc'
    DHL_EXPRESS_WORLDWIDE_DOC = 'dhl_express_worldwide_doc'
    DHL_EXPRESS_WORLDWIDE = 'dhl_express_worldwide'
    DHL_EXPRESS_WORLDWIDE_EU_DOC = 'dhl_express_worldwide_eu_doc'
    DHL_EXPRESS_BREAK_BULK_EXPRESS_DOC = 'dhl_express_break_bulk_express_doc'
    DHL_EXPRESS_EXPRESS_9_00_NONDOC = 'dhl_express_express_9_00_nondoc'
    DHL_EXPRESS_ECONOMY_SELECT_NONDOC = 'dhl_express_economy_select_nondoc'
    DHL_EXPRESS_BREAK_BULK_ECONOMY_DOC = 'dhl_express_break_bulk_economy_doc'
    DHL_EXPRESS_EXPRESS_9_00_DOC = 'dhl_express_express_9_00_doc'
    DHL_EXPRESS_EXPRESS_10_30_DOC = 'dhl_express_express_10_30_doc'
    DHL_EXPRESS_EXPRESS_10_30_NONDOC = 'dhl_express_express_10_30_nondoc'
    DHL_EXPRESS_EXPRESS_12_00_DOC = 'dhl_express_express_12_00_doc'
    DHL_EXPRESS_EUROPACK_NONDOC = 'dhl_express_europack_nondoc'
    DHL_EXPRESS_EXPRESS_ENVELOPE_DOC = 'dhl_express_express_envelope_doc'
    DHL_EXPRESS_EXPRESS_12_00_NONDOC = 'dhl_express_express_12_00_nondoc'
    DHL_EXPRESS_EXPRESS_12_DOC = 'dhl_express_express_12_doc'
    DHL_EXPRESS_WORLDWIDE_B2C_DOC = 'dhl_express_worldwide_b2c_doc'
    DHL_EXPRESS_WORLDWIDE_B2C_NONDOC = 'dhl_express_worldwide_b2c_nondoc'
    DHL_EXPRESS_MEDICAL_EXPRESS = 'dhl_express_medical_express'
    DHL_EXPRESS_EXPRESS_EASY_NONDOC = 'dhl_express_express_easy_nondoc'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelSendleEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | sendle_parcel | Sendle Parcel|
    | sendle_standard_dropoff | Sendle Standard Dropoff|
    """
    SENDLE_PARCEL = 'sendle_parcel'
    SENDLE_STANDARD_DROPOFF = 'sendle_standard_dropoff'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelGLSUSEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | gls_us_cps | GSO Ground|
    | gls_us_eps | Early Priority Overnight|
    | gls_us_ess | Early Saturday Delivery|
    | gls_us_nps | Noon Priority Overnight|
    | gls_us_pds | Priority Overnight|
    | gls_us_sds | Saturday Delivery|
    """
    GLS_US_CPS = 'gls_us_cps'
    GLS_US_EPS = 'gls_us_eps'
    GLS_US_ESS = 'gls_us_ess'
    GLS_US_NPS = 'gls_us_nps'
    GLS_US_PDS = 'gls_us_pds'
    GLS_US_SDS = 'gls_us_sds'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import requests as requests_http
from .sdkconfiguration import SDKConfiguration
from shippo import utils
from shippo._hooks import AfterErrorContext, AfterSuccessContext, BeforeRequestContext, HookContext
from shippo.models import components, errors, operations
from typing import Optional

class Shipments:
    r"""A shipment is the act of transporting goods. A shipment object contains **to** and **from** addresses, and the parcel details that you are shipping. You can use the shipment object to retrieve shipping rates and purchase a shipping label.
    <SchemaDefinition schemaRef=\"#/components/schemas/Shipment\"/>

    # Shipment Extras
    The following values are supported for the `extra` field of the shipment object.
    <SchemaDefinition schemaRef=\"#/components/schemas/ShipmentExtra\"/>
    """
    sdk_configuration: SDKConfiguration

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        self.sdk_configuration = sdk_config
        
    
    
    def list(self, request: operations.ListShipmentsRequest) -> components.ShipmentPaginatedList:
        r"""List all shipments
        Returns a list of all shipment objects.<br><br>
        In order to filter results, you must use the below path parameters. 
        A maximum date range of 90 days is permitted. 
        Provided dates should be ISO 8601 UTC dates (timezone offsets are currently not supported).<br><br>

        Optional path parameters:<br>
          `object_created_gt`- object(s) created greater than a provided date time<br>
          `object_created_gte` - object(s) created greater than or equal to a provided date time<br>
          `object_created_lt` - object(s) created less than a provided date time<br>
          `object_created_lte` - object(s) created less than or equal to a provided date time<br>

          Date format examples:<br>
            `2017-01-01`<br>
            `2017-01-01T03:30:30` or `2017-01-01T03:30:30.5`<br>
            `2017-01-01T03:30:30Z`<br><br>

          Example URL:<br>
            `https://api.goshippo.com/shipments/?object_created_gte=2017-01-01T00:00:30&object_created_lt=2017-04-01T00:00:30`
        """
        hook_ctx = HookContext(operation_id='ListShipments', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        _globals = operations.ListShipmentsGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/shipments'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        query_params = { **utils.get_query_params(request, _globals), **query_params }
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('GET', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.ShipmentPaginatedList])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def create(self, request: components.ShipmentCreateRequest) -> components.Shipment:
        r"""Create a new shipment
        Creates a new shipment object.
        """
        hook_ctx = HookContext(operation_id='CreateShipment', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        _globals = operations.CreateShipmentGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/shipments'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        req_content_type, data, form = utils.serialize_request_body(request, components.ShipmentCreateRequest, "request", False, False, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        if data is None and form is None:
            raise Exception('request body is required')
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('POST', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 201:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.Shipment])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def get(self, shipment_id: str) -> components.Shipment:
        r"""Retrieve a shipment
        Returns an existing shipment using an object ID
        """
        hook_ctx = HookContext(operation_id='GetShipment', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.GetShipmentRequest(
            shipment_id=shipment_id,
        )
        
        _globals = operations.GetShipmentGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(base_url, '/shipments/{ShipmentId}', request, _globals)
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('GET', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.Shipment])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from ...models.components import carriersenum as components_carriersenum
from typing import Optional


@dataclasses.dataclass
class ListCarrierAccountsGlobals:
    shippo_api_version: Optional[str] = dataclasses.field(default=None, metadata={'header': { 'field_name': 'SHIPPO-API-VERSION', 'style': 'simple', 'explode': False }})
    r"""Optional string used to pick a non-default API version to use. See our <a href=\\"https://docs.goshippo.com/docs/api_concepts/apiversioning/\\">API version</a> guide."""
    



@dataclasses.dataclass
class ListCarrierAccountsRequest:
    service_levels: Optional[bool] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'service_levels', 'style': 'form', 'explode': True }})
    r"""Appends the property `service_levels` to each returned carrier account"""
    carrier: Optional[components_carriersenum.CarriersEnum] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'carrier', 'style': 'form', 'explode': True }})
    r"""Filter the response by the specified carrier"""
    account_id: Optional[str] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'account_id', 'style': 'form', 'explode': True }})
    r"""Filter the response by the specified carrier account Id"""
    page: Optional[int] = dataclasses.field(default=1, metadata={'query_param': { 'field_name': 'page', 'style': 'form', 'explode': True }})
    r"""The page number you want to select"""
    results: Optional[int] = dataclasses.field(default=5, metadata={'query_param': { 'field_name': 'results', 'style': 'form', 'explode': True }})
    r"""The number of results to return per page (max 100, default 5)"""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelAustraliaPostEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | australia_post_express_post | Express Post|
    | australia_post_parcel_post | Parcel Post|
    | australia_post_pack_and_track_international | Pack and Track International|
    | australia_post_international_airmail | International Airmail|
    | australia_post_express_post_international | Express Post International|
    | australia_post_express_courier_international | Express Courier International|
    | australia_post_international_express | International Express|
    | australia_post_international_standard | International Standard|
    | australia_post_international_economy | International Economy|
    | australia_post_parcel_post_return | Parcel Post Return|
    """
    AUSTRALIA_POST_EXPRESS_POST = 'australia_post_express_post'
    AUSTRALIA_POST_PARCEL_POST = 'australia_post_parcel_post'
    AUSTRALIA_POST_PACK_AND_TRACK_INTERNATIONAL = 'australia_post_pack_and_track_international'
    AUSTRALIA_POST_INTERNATIONAL_AIRMAIL = 'australia_post_international_airmail'
    AUSTRALIA_POST_EXPRESS_POST_INTERNATIONAL = 'australia_post_express_post_international'
    AUSTRALIA_POST_EXPRESS_COURIER_INTERNATIONAL = 'australia_post_express_courier_international'
    AUSTRALIA_POST_INTERNATIONAL_EXPRESS = 'australia_post_international_express'
    AUSTRALIA_POST_INTERNATIONAL_STANDARD = 'australia_post_international_standard'
    AUSTRALIA_POST_INTERNATIONAL_ECONOMY = 'australia_post_international_economy'
    AUSTRALIA_POST_PARCEL_POST_RETURN = 'australia_post_parcel_post_return'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import requests as requests_http
from .sdkconfiguration import SDKConfiguration
from shippo import utils
from shippo._hooks import AfterErrorContext, AfterSuccessContext, BeforeRequestContext, HookContext
from shippo.models import components, errors, operations
from typing import Optional

class CarrierParcelTemplates:
    r"""A carrier parcel template represents a package used for shipping that has preset dimensions defined by a carrier. Some examples of a carrier parcel template include USPS Flat Rate Box and Fedex Small Pak. When using a carrier parcel template, the rates returned may be limited to the carrier that provides the box. You can create user parcel templates using a carrier parcel template. Shippo takes the dimensions of the carrier parcel template but you must configure the weight.

    <SchemaDefinition schemaRef=\"#/components/schemas/CarrierParcelTemplate\"/>
    """
    sdk_configuration: SDKConfiguration

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        self.sdk_configuration = sdk_config
        
    
    
    def list(self, include: Optional[operations.Include] = None, carrier: Optional[str] = None) -> components.CarrierParcelTemplateList:
        r"""List all carrier parcel templates
        List all carrier parcel template objects. <br> Use the following query string params to filter the results as needed. <br>
        <ul>
        <li>`include=all` (the default). Includes templates from all carriers </li>
        <li>`include=user`. Includes templates only from carriers which the user has added (whether or not they're currently enabled) </li>
        <li>`include=enabled`. includes templates only for carriers which the user has added and enabled </li>
        <li>`carrier=*token*`. filter by specific carrier, e.g. fedex, usps </li>
        </ul>
        """
        hook_ctx = HookContext(operation_id='ListCarrierParcelTemplates', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.ListCarrierParcelTemplatesRequest(
            include=include,
            carrier=carrier,
        )
        
        _globals = operations.ListCarrierParcelTemplatesGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/parcel-templates'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        query_params = { **utils.get_query_params(request, _globals), **query_params }
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('GET', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.CarrierParcelTemplateList])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def get(self, carrier_parcel_template_token: str) -> components.CarrierParcelTemplate:
        r"""Retrieve a carrier parcel templates
        Fetches the parcel template information for a specific carrier parcel template, identified by the token.
        """
        hook_ctx = HookContext(operation_id='GetCarrierParcelTemplate', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.GetCarrierParcelTemplateRequest(
            carrier_parcel_template_token=carrier_parcel_template_token,
        )
        
        _globals = operations.GetCarrierParcelTemplateGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(base_url, '/parcel-templates/{CarrierParcelTemplateToken}', request, _globals)
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('GET', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.CarrierParcelTemplate])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from .address import Address
from .customsdeclaration import CustomsDeclaration
from .parcel import Parcel
from .rate import Rate
from .responsemessage import ResponseMessage
from .shipmentextra import ShipmentExtra
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from enum import Enum
from shippo import utils
from typing import List, Optional


class ShipmentStatus(str, Enum):
    r"""`Waiting` shipments have been successfully submitted but not yet been processed.
    `Queued` shipments are currently being processed. 
    `Success` shipments have been processed successfully, meaning that rate generation has concluded. 
    `Error` does not occur currently and is reserved for future use.
    """
    ERROR = 'ERROR'
    QUEUED = 'QUEUED'
    SUCCESS = 'SUCCESS'
    WAITING = 'WAITING'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Shipment:
    r"""Shipment represents the parcel as retrieved from the database"""
    metadata: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata') }})
    r"""A string of up to 100 characters that can be filled with any additional information you want to attach to the object."""
    address_from: Address = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('address_from') }})
    r"""<a href=\\"#tag/Addresses\\">Address</a> object of the sender / seller. Will be returned expanded by default."""
    address_to: Address = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('address_to') }})
    r"""<a href=\\"#tag/Addresses\\">Address</a> object of the recipient / buyer. Will be returned expanded by default."""
    carrier_accounts: List[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('carrier_accounts') }})
    r"""An array of object_ids of the carrier account objects to be used for getting shipping rates for this shipment.
    If no carrier account object_ids are set in this field, Shippo will attempt to generate rates using all the 
    carrier accounts that have the `active` field set.
    """
    messages: List[ResponseMessage] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('messages') }})
    object_created: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_created'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""Date and time of Shipment creation."""
    object_id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_id') }})
    r"""Unique identifier of the given Shipment object."""
    object_owner: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_owner') }})
    r"""Username of the user who created the Shipment object."""
    object_updated: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_updated'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""Date and time of last Shipment update."""
    parcels: List[Parcel] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('parcels') }})
    r"""List of Parcel objects to be shipped."""
    rates: List[Rate] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('rates') }})
    r"""An array with all available rates. If <code>async</code> has been set to <code>false</code> in the request,
    this will be populated with all available rates in the response. Otherwise rates will be created
    asynchronously and this array will initially be empty.
    """
    status: ShipmentStatus = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status') }})
    r"""`Waiting` shipments have been successfully submitted but not yet been processed.
    `Queued` shipments are currently being processed. 
    `Success` shipments have been processed successfully, meaning that rate generation has concluded. 
    `Error` does not occur currently and is reserved for future use.
    """
    extra: Optional[ShipmentExtra] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('extra'), 'exclude': lambda f: f is None }})
    r"""An object holding optional extra services to be requested."""
    shipment_date: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('shipment_date'), 'exclude': lambda f: f is None }})
    r"""Date the shipment will be tendered to the carrier. Must be in the format `2014-01-18T00:35:03.463Z`.
    Defaults to current date and time if no value is provided. Please note that some carriers require this value to
    be in the future, on a working day, or similar.
    """
    address_return: Optional[Address] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('address_return'), 'exclude': lambda f: f is None }})
    r"""ID of the Address object where the shipment will be sent back to if it is not delivered
    (Only available for UPS, USPS, and Fedex shipments). <br/> 
    If this field is not set, your shipments will be returned to the address_from.
    """
    customs_declaration: Optional[CustomsDeclaration] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('customs_declaration'), 'exclude': lambda f: f is None }})
    test: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('test'), 'exclude': lambda f: f is None }})
    r"""Indicates whether the object has been created in test mode."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class AddressImporter:
    r"""Object that represents the address of the importer"""
    name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name'), 'exclude': lambda f: f is None }})
    r"""First and Last Name of the addressee"""
    company: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('company'), 'exclude': lambda f: f is None }})
    r"""Company Name"""
    street1: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('street1'), 'exclude': lambda f: f is None }})
    r"""First street line, 35 character limit. Usually street number and street name (except for DHL Germany, see street_no)."""
    street2: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('street2'), 'exclude': lambda f: f is None }})
    r"""Second street line, 35 character limit."""
    street3: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('street3'), 'exclude': lambda f: f is None }})
    r"""Third street line, 35 character limit.
    Only accepted for USPS international shipments, UPS domestic and UPS international shipments.
    """
    street_no: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('street_no'), 'exclude': lambda f: f is None }})
    r"""Street number of the addressed building.
    This field can be included in street1 for all carriers except for DHL Germany.
    """
    city: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('city'), 'exclude': lambda f: f is None }})
    r"""Name of a city"""
    state: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('state'), 'exclude': lambda f: f is None }})
    r"""**required for purchase for some countries**<br>
    State/Province values are required for shipments from/to the US, AU, and CA. UPS requires province for some 
    countries (i.e Ireland). To receive more accurate quotes, passing this field is recommended. Most carriers 
    only accept two or three character state abbreviations.
    """
    zip: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('zip'), 'exclude': lambda f: f is None }})
    r"""Postal code of an Address."""
    country: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('country'), 'exclude': lambda f: f is None }})
    r"""Example: `US` or `DE`. All accepted values can be found on the
    <a href=\"http://www.iso.org/\" target=\"blank\">Official ISO Website</a>.
    Sending a country is always required.
    """
    phone: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('phone'), 'exclude': lambda f: f is None }})
    r"""Addresses containing a phone number allow carriers to call the recipient when delivering the Parcel. This
    increases the probability of delivery and helps to avoid accessorial charges after a Parcel has been shipped.
    """
    email: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('email'), 'exclude': lambda f: f is None }})
    r"""E-mail address of the contact person, RFC3696/5321-compliant."""
    is_residential: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('is_residential'), 'exclude': lambda f: f is None }})
    r"""Indicates whether the address provided is a residential address or not."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class DryIce:
    r"""Specify that the package contains Dry Ice (FedEx, Veho, and UPS only)."""
    contains_dry_ice: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('contains_dry_ice'), 'exclude': lambda f: f is None }})
    r"""Mandatory. Specifies that the package contains Dry Ice."""
    weight: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('weight'), 'exclude': lambda f: f is None }})
    r"""Mandatory. Units must be in Kilograms. Cannot be greater than package weight."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .webhookeventtypeenum import WebhookEventTypeEnum
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class WebhookUpdateRequest:
    event: WebhookEventTypeEnum = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event') }})
    r"""Type of event that triggered the webhook."""
    url: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('url') }})
    r"""URL webhook events are sent to."""
    active: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('active'), 'exclude': lambda f: f is None }})
    r"""Determines whether the webhook is active or not."""
    is_test: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('is_test'), 'exclude': lambda f: f is None }})
    r"""Determines whether the webhook is a test webhook or not."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelColissimoEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | colissimo_home | Domicile|
    | colissimo_pick_up_point | Point Retrait|
    | colissimo_return_mainland_france | Retour France|
    """
    COLISSIMO_HOME = 'colissimo_home'
    COLISSIMO_PICK_UP_POINT = 'colissimo_pick_up_point'
    COLISSIMO_RETURN_MAINLAND_FRANCE = 'colissimo_return_mainland_france'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import requests as requests_http
from .sdkconfiguration import SDKConfiguration
from shippo import utils
from shippo._hooks import AfterErrorContext, AfterSuccessContext, BeforeRequestContext, HookContext
from shippo.models import components, errors, operations
from typing import List, Optional

class ServiceGroups:
    r"""A service group is a set of service levels grouped together.
    Rates at checkout uses services groups to present available shipping options to customers in their shopping basket.
    <SchemaDefinition schemaRef=\"#/components/schemas/ServiceGroup\"/>
    """
    sdk_configuration: SDKConfiguration

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        self.sdk_configuration = sdk_config
        
    
    
    def list(self) -> List[components.ServiceGroup]:
        r"""List all service groups
        Returns a list of service group objects.
        """
        hook_ctx = HookContext(operation_id='ListServiceGroups', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.ListServiceGroupsRequest(
        )
        
        _globals = operations.ListServiceGroupsGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/service-groups'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('GET', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[List[components.ServiceGroup]])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def create(self, request: components.ServiceGroupCreateRequest) -> components.ServiceGroup:
        r"""Create a new service group
        Creates a new service group.
        """
        hook_ctx = HookContext(operation_id='CreateServiceGroup', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        _globals = operations.CreateServiceGroupGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/service-groups'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        req_content_type, data, form = utils.serialize_request_body(request, components.ServiceGroupCreateRequest, "request", False, False, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        if data is None and form is None:
            raise Exception('request body is required')
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('POST', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 201:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.ServiceGroup])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def update(self, request: Optional[components.ServiceGroupUpdateRequest] = None) -> components.ServiceGroup:
        r"""Update an existing service group
        Updates an existing service group object. <br>The object_id cannot be updated as it is the unique identifier for the object.
        """
        hook_ctx = HookContext(operation_id='UpdateServiceGroup', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        _globals = operations.UpdateServiceGroupGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/service-groups'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        req_content_type, data, form = utils.serialize_request_body(request, Optional[components.ServiceGroupUpdateRequest], "request", False, True, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('PUT', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.ServiceGroup])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def delete(self, service_group_id: str):
        r"""Delete a service group
        Deletes an existing service group using an object ID.
        """
        hook_ctx = HookContext(operation_id='DeleteServiceGroup', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.DeleteServiceGroupRequest(
            service_group_id=service_group_id,
        )
        
        _globals = operations.DeleteServiceGroupGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(base_url, '/service-groups/{ServiceGroupId}', request, _globals)
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        headers['Accept'] = '*/*'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('DELETE', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 204:
            pass
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelPostItalianeEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | poste_italiane_delivery_business_express | Poste Delivery Business Express|
    """
    POSTE_ITALIANE_DELIVERY_BUSINESS_EXPRESS = 'poste_italiane_delivery_business_express'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelDeutschePostEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | deutsche_post_postkarte | Postkarte|
    | deutsche_post_standardbrief | Standardbrief|
    | deutsche_post_kompaktbrief | Kompaktbrief|
    | deutsche_post_grossbrief | Grossbrief|
    | deutsche_post_maxibrief | Maxibrief|
    | deutsche_post_maxibrief_plus | Maxibrief Plus|
    | deutsche_post_warenpost_international_xs | Warenpost International XS|
    | deutsche_post_warenpost_international_s | Warenpost International S|
    | deutsche_post_warenpost_international_m | Warenpost International M|
    | deutsche_post_warenpost_international_l | Warenpost International L|
    """
    DEUTSCHE_POST_POSTKARTE = 'deutsche_post_postkarte'
    DEUTSCHE_POST_STANDARDBRIEF = 'deutsche_post_standardbrief'
    DEUTSCHE_POST_KOMPAKTBRIEF = 'deutsche_post_kompaktbrief'
    DEUTSCHE_POST_GROSSBRIEF = 'deutsche_post_grossbrief'
    DEUTSCHE_POST_MAXIBRIEF = 'deutsche_post_maxibrief'
    DEUTSCHE_POST_MAXIBRIEF_PLUS = 'deutsche_post_maxibrief_plus'
    DEUTSCHE_POST_WARENPOST_INTERNATIONAL_XS = 'deutsche_post_warenpost_international_xs'
    DEUTSCHE_POST_WARENPOST_INTERNATIONAL_S = 'deutsche_post_warenpost_international_s'
    DEUTSCHE_POST_WARENPOST_INTERNATIONAL_M = 'deutsche_post_warenpost_international_m'
    DEUTSCHE_POST_WARENPOST_INTERNATIONAL_L = 'deutsche_post_warenpost_international_l'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from ...models.components import webhookupdaterequest as components_webhookupdaterequest


@dataclasses.dataclass
class UpdateWebhookRequest:
    webhook_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'webhookId', 'style': 'simple', 'explode': False }})
    r"""Object ID of the webhook to retrieve"""
    webhook_update_request: components_webhookupdaterequest.WebhookUpdateRequest = dataclasses.field(metadata={'request': { 'media_type': 'application/json' }})
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelAramexAustraliaEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | fastway_australia_parcel | Parcel|
    | fastway_australia_satchel | Satchel|
    | fastway_australia_box_small | Box Small|
    | fastway_australia_box_medium | Box Medium|
    | fastway_australia_box_large | Box Large|
    """
    FASTWAY_AUSTRALIA_PARCEL = 'fastway_australia_parcel'
    FASTWAY_AUSTRALIA_SATCHEL = 'fastway_australia_satchel'
    FASTWAY_AUSTRALIA_BOX_SMALL = 'fastway_australia_box_small'
    FASTWAY_AUSTRALIA_BOX_MEDIUM = 'fastway_australia_box_medium'
    FASTWAY_AUSTRALIA_BOX_LARGE = 'fastway_australia_box_large'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from .location import Location
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from enum import Enum
from shippo import utils
from typing import List, Optional


class PickupStatus(str, Enum):
    r"""Indicates the status of the pickup."""
    PENDING = 'PENDING'
    CONFIRMED = 'CONFIRMED'
    ERROR = 'ERROR'
    CANCELLED = 'CANCELLED'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Pickup:
    carrier_account: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('carrier_account') }})
    r"""The object ID of your USPS or DHL Express carrier account.
    You can retrieve this from your Rate requests or our <a href=\"#tag/Carrier-Accounts/\">Carrier Accounts</a> endpoint.
    """
    location: Location = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('location') }})
    r"""Location where the parcel(s) will be picked up."""
    requested_end_time: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('requested_end_time'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""The latest that you requested your parcels to be available for pickup.
    Expressed in the timezone specified in the response.
    """
    requested_start_time: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('requested_start_time'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""The earliest that you requested your parcels to be ready for pickup.
    Expressed in the timezone specified in the response.
    """
    transactions: List[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('transactions') }})
    r"""The transaction(s) object ID(s) for the parcel(s) that need to be picked up."""
    metadata: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    r"""A string of up to 100 characters that can be filled with any additional information you
    want to attach to the object.
    """
    object_created: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_created'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""Date and time of Pickup creation."""
    object_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_id'), 'exclude': lambda f: f is None }})
    r"""Unique identifier of the given Pickup object."""
    object_updated: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_updated'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""Date and time of last Pickup update."""
    confirmed_start_time: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('confirmed_start_time'), 'exclude': lambda f: f is None }})
    r"""The earliest that your parcels will be ready for pickup, confirmed by the carrier.
    Expressed in the timezone specified in the response.
    """
    confirmed_end_time: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('confirmed_end_time'), 'exclude': lambda f: f is None }})
    r"""The latest that your parcels will be available for pickup, confirmed by the carrier.
    Expressed in the timezone specified in the response.
    """
    cancel_by_time: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('cancel_by_time'), 'exclude': lambda f: f is None }})
    r"""The latest time to cancel a pickup. Expressed in the timezone specified in the response.
    To cancel a pickup, you will need to contact the carrier directly. 
    The ability to cancel a pickup through Shippo may be released in future iterations.
    """
    status: Optional[PickupStatus] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status'), 'exclude': lambda f: f is None }})
    r"""Indicates the status of the pickup."""
    confirmation_code: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('confirmation_code'), 'exclude': lambda f: f is None }})
    r"""Pickup's confirmation code returned by the carrier.
    To edit or cancel a pickup, you will need to contact USPS or DHL Express directly and provide your `confirmation_code`.
    """
    timezone: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('timezone'), 'exclude': lambda f: f is None }})
    r"""The pickup time windows will be in the time zone specified here, not UTC."""
    messages: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('messages'), 'exclude': lambda f: f is None }})
    r"""An array containing strings of any messages generated during validation."""
    is_test: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('is_test'), 'exclude': lambda f: f is None }})
    r"""Indicates whether the object has been created in test mode."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ShipmentExtraLasershipAttributesEnum(str, Enum):
    TWO_PERSON_DELIVERY = 'TwoPersonDelivery'
    EXPLOSIVE = 'Explosive'
    ALCOHOL = 'Alcohol'
    HAZMAT = 'Hazmat'
    CONTROLLED_SUBSTANCE = 'ControlledSubstance'
    REFRIGERATED = 'Refrigerated'
    DRY_ICE = 'DryIce'
    PERISHABLE = 'Perishable'
    NO_RTS = 'NoRTS'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import requests as requests_http
from .sdkconfiguration import SDKConfiguration
from shippo import utils
from shippo._hooks import AfterErrorContext, AfterSuccessContext, BeforeRequestContext, HookContext
from shippo.models import components, errors, operations
from typing import Optional

class ShippoAccounts:
    r"""Shippo Accounts are used by Shippo Platform Accounts to create and manage Managed Shippo Accounts.
    Managed Shippo Accounts are headless accounts that represent your customers. They are opaque to your end customers, meaning customers do not need to create their own Shippo login or have a billing relationship with Shippo. 
    They can be used by marketplaces, e-commerce platforms, and third-party logistics providers who want to offer, seamless, built-in shipping functionality to their customers. See our <a href=\"https://docs.goshippo.com/docs/platformaccounts/platform_accounts/\">guide</a> for more details.
    <SchemaDefinition schemaRef=\"#/components/schemas/ShippoAccount\"/>
    """
    sdk_configuration: SDKConfiguration

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        self.sdk_configuration = sdk_config
        
    
    
    def list(self, page: Optional[int] = None, results: Optional[int] = None) -> components.ShippoAccountPaginatedList:
        r"""List all Shippo Accounts
        Returns a list of Shippo Managed Accounts objects.
        """
        hook_ctx = HookContext(operation_id='ListShippoAccounts', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.ListShippoAccountsRequest(
            page=page,
            results=results,
        )
        
        _globals = operations.ListShippoAccountsGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/shippo-accounts'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        query_params = { **utils.get_query_params(request, _globals), **query_params }
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('GET', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.ShippoAccountPaginatedList])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def create(self, request: components.ShippoAccountUpdateRequest) -> components.ShippoAccount:
        r"""Create a Shippo Account
        Creates a new <a href=\"https://docs.goshippo.com/docs/platformaccounts/platform_using_accounts/\">Shippo Managed Account</a>.
        """
        hook_ctx = HookContext(operation_id='CreateShippoAccount', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        _globals = operations.CreateShippoAccountGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/shippo-accounts'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        req_content_type, data, form = utils.serialize_request_body(request, components.ShippoAccountUpdateRequest, "request", False, False, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        if data is None and form is None:
            raise Exception('request body is required')
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('POST', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 201:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.ShippoAccount])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def get(self, shippo_account_id: str) -> components.ShippoAccount:
        r"""Retrieve a Shippo Account
        Returns a Shippo Managed Account using an object ID.
        """
        hook_ctx = HookContext(operation_id='GetShippoAccount', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.GetShippoAccountRequest(
            shippo_account_id=shippo_account_id,
        )
        
        _globals = operations.GetShippoAccountGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(base_url, '/shippo-accounts/{ShippoAccountId}', request, _globals)
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('GET', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.ShippoAccount])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def update(self, shippo_account_id: str, shippo_account_update_request: Optional[components.ShippoAccountUpdateRequest] = None) -> components.ShippoAccount:
        r"""Update a Shippo Account
        Updates a Shippo Managed Account using an object ID.
        """
        hook_ctx = HookContext(operation_id='UpdateShippoAccount', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.UpdateShippoAccountRequest(
            shippo_account_id=shippo_account_id,
            shippo_account_update_request=shippo_account_update_request,
        )
        
        _globals = operations.UpdateShippoAccountGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(base_url, '/shippo-accounts/{ShippoAccountId}', request, _globals)
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        req_content_type, data, form = utils.serialize_request_body(request, operations.UpdateShippoAccountRequest, "shippo_account_update_request", False, True, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('PUT', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.ShippoAccount])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelChronopostEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | chronopost_13 | Chrono 13|
    | chronopost_10 | Chrono 10|
    | chronopost_18| Chrono 18|
    | chronopost_relais_fr | Chrono Point Relais|
    | chronopost_classic | Chrono Classic International|
    | chronopost_express | Chrono Express International|
    """
    CHRONOPOST_13 = 'chronopost_13'
    CHRONOPOST_10 = 'chronopost_10'
    CHRONOPOST_18 = 'chronopost_18'
    CHRONOPOST_RELAIS_FR = 'chronopost_relais_fr'
    CHRONOPOST_CLASSIC = 'chronopost_classic'
    CHRONOPOST_EXPRESS = 'chronopost_express'



from contextvars import ContextVar
from typing import Optional

from requests import Session, PreparedRequest, Response


class DebugSession(Session):

    _last_request: ContextVar
    _last_response: ContextVar

    def __init__(self):
        super().__init__()
        self._last_request = ContextVar("last_request", default=None)
        self._last_response = ContextVar("last_response", default=None)

    def clear(self):
        self._last_request.set(None)
        self._last_response.set(None)

    @property
    def last_request(self) -> Optional[PreparedRequest]:
        return self._last_request.get()

    @property
    def last_response(self) -> Optional[Response]:
        return self._last_response.get()

    def send(self, request, **kwargs):
        self._last_request.set(request)
        self._last_response.set(None)
        response = Session.send(self, request, **kwargs)
        self._last_response.set(response)
        return response



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelOnTracEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | ontrac_ground | Ground|
    | ontrac_sunrise_gold | Sunrise Gold|
    | ontrac_sunrise | Sunrise|
    """
    ONTRAC_GROUND = 'ontrac_ground'
    ONTRAC_SUNRISE_GOLD = 'ontrac_sunrise_gold'
    ONTRAC_SUNRISE = 'ontrac_sunrise'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class WeightUnitEnum(str, Enum):
    r"""The unit used for weight."""
    G = 'g'
    KG = 'kg'
    LB = 'lb'
    OZ = 'oz'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelAPCPostalEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | apc_postal_parcelconnect_expedited | parcelConnect Expedited|
    | apc_postal_parcelconnect_priority | parcelConnect Priority|
    | apc_postal_parcelconnect_priority_delcon | parcelConnect Priority Delcon|
    | apc_postal_parcelconnect_priority_pqw | parcelConnect Priority PQW|
    | apc_postal_parcelconnect_book_service | parcelConnect Book Service|
    | apc_postal_parcelconnect_standard | parcelConnect Standard|
    | apc_postal_parcelconnect_epmi | parcelConnect ePMI|
    | apc_postal_parcelconnect_epacket | parcelConnect ePacket|
    | apc_postal_parcelconnect_epmei | parcelConnect ePMEI|
    """
    APC_POSTAL_PARCELCONNECT_EXPEDITED = 'apc_postal_parcelconnect_expedited'
    APC_POSTAL_PARCELCONNECT_PRIORITY = 'apc_postal_parcelconnect_priority'
    APC_POSTAL_PARCELCONNECT_PRIORITY_DELCON = 'apc_postal_parcelconnect_priority_delcon'
    APC_POSTAL_PARCELCONNECT_PRIORITY_PQW = 'apc_postal_parcelconnect_priority_pqw'
    APC_POSTAL_PARCELCONNECT_BOOK_SERVICE = 'apc_postal_parcelconnect_book_service'
    APC_POSTAL_PARCELCONNECT_STANDARD = 'apc_postal_parcelconnect_standard'
    APC_POSTAL_PARCELCONNECT_EPMI = 'apc_postal_parcelconnect_epmi'
    APC_POSTAL_PARCELCONNECT_EPACKET = 'apc_postal_parcelconnect_epacket'
    APC_POSTAL_PARCELCONNECT_EPMEI = 'apc_postal_parcelconnect_epmei'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .distanceunitenum import DistanceUnitEnum
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CarrierParcelTemplate:
    carrier: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('carrier'), 'exclude': lambda f: f is None }})
    r"""The name of the carrier that provides this parcel template"""
    distance_unit: Optional[DistanceUnitEnum] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('distance_unit'), 'exclude': lambda f: f is None }})
    r"""The measure unit used for length, width and height."""
    height: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('height'), 'exclude': lambda f: f is None }})
    r"""The height of the package, in units specified by the distance_unit attribute"""
    is_variable_dimensions: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('is_variable_dimensions'), 'exclude': lambda f: f is None }})
    r"""True if the carrier parcel template allows custom dimensions, such as USPS Softpack."""
    length: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('length'), 'exclude': lambda f: f is None }})
    r"""The length of the package, in units specified by the distance_unit attribute"""
    name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name'), 'exclude': lambda f: f is None }})
    r"""The name of the carrier parcel template"""
    token: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('token'), 'exclude': lambda f: f is None }})
    r"""The unique string representation of the carrier parcel template"""
    width: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('width'), 'exclude': lambda f: f is None }})
    r"""The width of the package, in units specified by the distance_unit attribute"""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .address import Address
from .lineitem import LineItem
from .ordershopappenum import OrderShopAppEnum
from .orderstatusenum import OrderStatusEnum
from .weightunitenum import WeightUnitEnum
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import List, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Order:
    placed_at: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('placed_at') }})
    r"""Date and time when the order was placed. This datetime can be different from the datetime of the order object creation on Shippo."""
    to_address: Address = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('to_address') }})
    r"""<a href=\\"#tag/Addresses\\">Address</a> object of the recipient / buyer. Will be returned expanded by default."""
    currency: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('currency'), 'exclude': lambda f: f is None }})
    r"""**Required if total_price is provided**<br>
    Currency of the <code>total_price</code> and <code>total_tax</code> amounts.
    """
    notes: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('notes'), 'exclude': lambda f: f is None }})
    r"""Custom buyer- or seller-provided notes about the order."""
    order_number: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('order_number'), 'exclude': lambda f: f is None }})
    r"""An alphanumeric identifier for the order used by the seller/buyer. This identifier doesn't need to be unique."""
    order_status: Optional[OrderStatusEnum] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('order_status'), 'exclude': lambda f: f is None }})
    r"""Current state of the order. See the <a href=\\"https://docs.goshippo.com/docs/orders/orders/\\">orders tutorial</a>
    for the logic of how the status is handled.
    """
    shipping_cost: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('shipping_cost'), 'exclude': lambda f: f is None }})
    r"""Amount paid by the buyer for shipping. This amount can be different from the price the seller will actually pay for shipping."""
    shipping_cost_currency: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('shipping_cost_currency'), 'exclude': lambda f: f is None }})
    r"""**Required if shipping_cost is provided**<br>
    Currency of the <code>shipping_cost</code> amount.
    """
    shipping_method: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('shipping_method'), 'exclude': lambda f: f is None }})
    r"""Shipping method (carrier + service or other free text description) chosen by the buyer.
    This value can be different from the shipping method the seller will actually choose.
    """
    subtotal_price: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('subtotal_price'), 'exclude': lambda f: f is None }})
    total_price: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('total_price'), 'exclude': lambda f: f is None }})
    r"""Total amount paid by the buyer for this order."""
    total_tax: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('total_tax'), 'exclude': lambda f: f is None }})
    r"""Total tax amount paid by the buyer for this order."""
    weight: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('weight'), 'exclude': lambda f: f is None }})
    r"""Total weight of the order."""
    weight_unit: Optional[WeightUnitEnum] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('weight_unit'), 'exclude': lambda f: f is None }})
    r"""The unit used for weight."""
    from_address: Optional[Address] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('from_address'), 'exclude': lambda f: f is None }})
    r"""<a href=\\"#tag/Addresses\\">Address</a> object of the sender / seller. Will be returned expanded by default."""
    line_items: Optional[List[LineItem]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('line_items'), 'exclude': lambda f: f is None }})
    r"""Array of <a href=\\"#section/Line-Item\\">line item</a> objects representing the items in this order.
    All objects will be returned expanded by default.
    """
    object_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_id'), 'exclude': lambda f: f is None }})
    r"""Unique identifier of the order object."""
    object_owner: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_owner'), 'exclude': lambda f: f is None }})
    r"""Username of the user who created the object."""
    shop_app: Optional[OrderShopAppEnum] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('shop_app'), 'exclude': lambda f: f is None }})
    r"""Platform the order was created on and, if applicable, imported from.
    Orders created via the Shippo API or dashboard will have the value \"Shippo\".
    """
    transactions: Optional[List[str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('transactions'), 'exclude': lambda f: f is None }})
    r"""Array of <a href=\\"#tag/Transactions\\">transaction</a> objects representing all shipping labels purchased for this order.
    All objects are returned expanded with a limited number of fields by default.
    """
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses


@dataclasses.dataclass
class DeleteWebhookRequest:
    webhook_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'webhookId', 'style': 'simple', 'explode': False }})
    r"""Object ID of the webhook to delete"""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ShipmentExtraReturnServiceTypeUPSEnum(str, Enum):
    PRINT_AND_MAIL = 'PRINT_AND_MAIL'
    ATTEMPT_1 = 'ATTEMPT_1'
    ATTEMPT_3 = 'ATTEMPT_3'
    ELECTRONIC_LABEL = 'ELECTRONIC_LABEL'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelAirterraEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | airterra_ground | GroundXC|
    | airterra_postal | FastPost|
    """
    AIRTERRA_GROUND = 'airterra_ground'
    AIRTERRA_POSTAL = 'airterra_postal'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelRoyalMailEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | royal_mail_tracked_letter_boxable_24_no_signature | Royal Mail Tracked Letter-Boxable 24 No Signature|
    | royal_mail_tracked_letter_boxable_48_no_signature | Royal Mail Tracked Letter-Boxable 48 No Signature|
    | royal_mail_tracked_24_returns | Royal Mail Tracked Returns 24|
    | royal_mail_tracked_48_returns | Royal Mail Tracked Returns 48|
    | royal_mail_special_delivery_guaranteed_1pm | Special Delivery Guaranteed by 1pm
    | royal_mail_special_delivery_guaranteed_9am | Special Delivery Guaranteed by 9am
    | royal_mail_24 | RoyalMail 24
    | royal_mail_48 | RoyalMail 48
    | royal_mail_signed_for_first_class | Signed for First Class
    | royal_mail_signed_for_second_class | Signed for Second Class
    | royal_mail_standard_first_class | Standard First Class
    | royal_mail_standard_second_class | Standard Second Class
    | royal_mail_tracked_24_no_signature | Royal Mail Tracked 24 No Signature
    | royal_mail_tracked_48_no_signature | Royal Mail Ttacked 48 No Signature
    | royal_mail_intl_bus_mail_lrg_ltr_zone_sort_pri | International Business Mail Large Letter Zone Sort Priority
    | royal_mail_intl_bus_parcels_tracked_zone_sort | International Business Parcels Tracked Zone Sort
    | royal_mail_intl_bus_parcels_tracked_country_priced | International Business Parcels Tracked Country Priced
    | royal_mail_intl_bus_parcels_tracked_signed_zone_srt | International Business Parcels & Signed Zone Sort
    | royal_mail_24_flat_rate | Royal Mail 24 Parcel Flat Rate
    | royal_mail_48_flat_rate | Royal Mail 48 Parcel Flat Rate
    """
    ROYAL_MAIL_TRACKED_LETTER_BOXABLE_24_NO_SIGNATURE = 'royal_mail_tracked_letter_boxable_24_no_signature'
    ROYAL_MAIL_TRACKED_LETTER_BOXABLE_48_NO_SIGNATURE = 'royal_mail_tracked_letter_boxable_48_no_signature'
    ROYAL_MAIL_TRACKED_24_RETURNS = 'royal_mail_tracked_24_returns'
    ROYAL_MAIL_TRACKED_48_RETURNS = 'royal_mail_tracked_48_returns'
    ROYAL_MAIL_SPECIAL_DELIVERY_GUARANTEED_1PM = 'royal_mail_special_delivery_guaranteed_1pm'
    ROYAL_MAIL_SPECIAL_DELIVERY_GUARANTEED_9AM = 'royal_mail_special_delivery_guaranteed_9am'
    ROYAL_MAIL_24 = 'royal_mail_24'
    ROYAL_MAIL_48 = 'royal_mail_48'
    ROYAL_MAIL_SIGNED_FOR_FIRST_CLASS = 'royal_mail_signed_for_first_class'
    ROYAL_MAIL_SIGNED_FOR_SECOND_CLASS = 'royal_mail_signed_for_second_class'
    ROYAL_MAIL_STANDARD_FIRST_CLASS = 'royal_mail_standard_first_class'
    ROYAL_MAIL_STANDARD_SECOND_CLASS = 'royal_mail_standard_second_class'
    ROYAL_MAIL_TRACKED_24_NO_SIGNATURE = 'royal_mail_tracked_24_no_signature'
    ROYAL_MAIL_TRACKED_48_NO_SIGNATURE = 'royal_mail_tracked_48_no_signature'
    ROYAL_MAIL_INTL_BUS_MAIL_LRG_LTR_ZONE_SORT_PRI = 'royal_mail_intl_bus_mail_lrg_ltr_zone_sort_pri'
    ROYAL_MAIL_INTL_BUS_PARCELS_TRACKED_ZONE_SORT = 'royal_mail_intl_bus_parcels_tracked_zone_sort'
    ROYAL_MAIL_INTL_BUS_PARCELS_TRACKED_COUNTRY_PRICED = 'royal_mail_intl_bus_parcels_tracked_country_priced'
    ROYAL_MAIL_INTL_BUS_PARCELS_TRACKED_SIGNED_ZONE_SRT = 'royal_mail_intl_bus_parcels_tracked_signed_zone_srt'
    ROYAL_MAIL_24_FLAT_RATE = 'royal_mail_24_flat_rate'
    ROYAL_MAIL_48_FLAT_RATE = 'royal_mail_48_flat_rate'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from shippo import utils


@dataclasses.dataclass
class CarrierAccountFedExCreateRequestParameters:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CarrierAccountFedExCreateRequest:
    carrier: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('carrier') }})
    parameters: CarrierAccountFedExCreateRequestParameters = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('parameters') }})
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from .distanceunitenum import DistanceUnitEnum
from .parcelextra import ParcelExtra
from .parceltemplateenumset import ParcelTemplateEnumSet
from .weightunitenum import WeightUnitEnum
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from enum import Enum
from shippo import utils
from typing import Optional


class ObjectState(str, Enum):
    r"""A Parcel will only be valid when all required values have been sent and validated successfully."""
    VALID = 'VALID'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Parcel:
    mass_unit: WeightUnitEnum = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('mass_unit') }})
    r"""The unit used for weight."""
    weight: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('weight') }})
    r"""Weight of the parcel. Up to six digits in front and four digits after the decimal separator are accepted."""
    distance_unit: DistanceUnitEnum = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('distance_unit') }})
    r"""The measure unit used for length, width and height."""
    height: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('height') }})
    r"""Height of the parcel. Up to six digits in front and four digits after the decimal separator are accepted."""
    length: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('length') }})
    r"""Length of the Parcel. Up to six digits in front and four digits after the decimal separator are accepted."""
    width: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('width') }})
    r"""Width of the Parcel. Up to six digits in front and four digits after the decimal separator are accepted."""
    extra: Optional[ParcelExtra] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('extra'), 'exclude': lambda f: f is None }})
    r"""An object holding optional extra services to be requested for each parcel in a multi-piece shipment.
    See the <a href=\"#section/Parcel-Extras\">Parcel Extra table below</a> for all available services.
    """
    metadata: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    object_created: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_created'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""Date and time of Parcel creation."""
    object_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_id'), 'exclude': lambda f: f is None }})
    r"""Unique identifier of the given Parcel object. This ID is required to create a Shipment object."""
    object_owner: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_owner'), 'exclude': lambda f: f is None }})
    r"""Username of the user who created the Parcel object."""
    object_state: Optional[ObjectState] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_state'), 'exclude': lambda f: f is None }})
    r"""A Parcel will only be valid when all required values have been sent and validated successfully."""
    object_updated: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_updated'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""Date and time of last Parcel update. Since you cannot update Parcels after they were created, this time stamp reflects the time when the Parcel was changed by Shippo's systems for the last time, e.g., during sorting the dimensions given."""
    template: Optional[ParcelTemplateEnumSet] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('template'), 'exclude': lambda f: f is None }})
    r"""If template is passed, `length`, `width`, `height`, and `distance_unit` are not required"""
    test: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('test'), 'exclude': lambda f: f is None }})
    r"""Indicates whether the object has been created in test mode."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ParcelTemplateUPSEnum(str, Enum):
    r"""|Token | Name | Dimensions|
    |:---|:---|:---|
    | UPS_Box_10kg | Box 10kg | 410.00 x 335.00 x 265.00 mm|
    | UPS_Box_25kg | Box 25kg | 484.00 x 433.00 x 350.00 mm|
    | UPS_Express_Box | Express Box | 460.00 x 315.00 x 95.00 mm|
    | UPS_Express_Box_Large | Express Box Large | 18.00 x 13.00 x 3.00 in|
    | UPS_Express_Box_Medium | Express Box Medium | 15.00 x 11.00 x 3.00 in|
    | UPS_Express_Box_Small | Express Box Small | 13.00 x 11.00 x 2.00 in|
    | UPS_Express_Envelope | Express Envelope | 12.50 x 9.50 x 2.00 in|
    | UPS_Express_Hard_Pak | Express Hard Pak | 14.75 x 11.50 x 2.00 in|
    | UPS_Express_Legal_Envelope | Express Legal Envelope | 15.00 x 9.50 x 2.00 in|
    | UPS_Express_Pak | Express Pak | 16.00 x 12.75 x 2.00 in|
    | UPS_Express_Tube | Express Tube | 970.00 x 190.00 x 165.00 mm|
    | UPS_Laboratory_Pak | Laboratory Pak | 17.25 x 12.75 x 2.00 in|
    | UPS_MI_BPM | BPM (Mail Innovations - Domestic &amp; International) | 0.00 x 0.00 x 0.00 in|
    | UPS_MI_BPM_Flat | BPM Flat (Mail Innovations - Domestic &amp; International) | 0.00 x 0.00 x 0.00 in|
    | UPS_MI_BPM_Parcel | BPM Parcel (Mail Innovations - Domestic &amp; International) | 0.00 x 0.00 x 0.00 in|
    | UPS_MI_First_Class | First Class (Mail Innovations - Domestic only) | 0.00 x 0.00 x 0.00 in|
    | UPS_MI_Flat | Flat (Mail Innovations - Domestic only) | 0.00 x 0.00 x 0.00 in|
    | UPS_MI_Irregular | Irregular (Mail Innovations - Domestic only) | 0.00 x 0.00 x 0.00 in|
    | UPS_MI_Machinable | Machinable (Mail Innovations - Domestic only) | 0.00 x 0.00 x 0.00 in|
    | UPS_MI_MEDIA_MAIL | Media Mail (Mail Innovations - Domestic only) | 0.00 x 0.00 x 0.00 in|
    | UPS_MI_Parcel_Post | Parcel Post (Mail Innovations - Domestic only) | 0.00 x 0.00 x 0.00 in|
    | UPS_MI_Priority | Priority (Mail Innovations - Domestic only) | 0.00 x 0.00 x 0.00 in|
    | UPS_MI_Standard_Flat | Standard Flat (Mail Innovations - Domestic only) | 0.00 x 0.00 x 0.00 in|
    | UPS_Pad_Pak | Pad Pak | 14.75 x 11.00 x 2.00 in|
    | UPS_Pallet | Pallet | 120.00 x 80.00 x 200.00 cm|
    """
    UPS_BOX_10KG = 'UPS_Box_10kg'
    UPS_BOX_25KG = 'UPS_Box_25kg'
    UPS_EXPRESS_BOX = 'UPS_Express_Box'
    UPS_EXPRESS_BOX_LARGE = 'UPS_Express_Box_Large'
    UPS_EXPRESS_BOX_MEDIUM = 'UPS_Express_Box_Medium'
    UPS_EXPRESS_BOX_SMALL = 'UPS_Express_Box_Small'
    UPS_EXPRESS_ENVELOPE = 'UPS_Express_Envelope'
    UPS_EXPRESS_HARD_PAK = 'UPS_Express_Hard_Pak'
    UPS_EXPRESS_LEGAL_ENVELOPE = 'UPS_Express_Legal_Envelope'
    UPS_EXPRESS_PAK = 'UPS_Express_Pak'
    UPS_EXPRESS_TUBE = 'UPS_Express_Tube'
    UPS_LABORATORY_PAK = 'UPS_Laboratory_Pak'
    UPS_MI_BPM = 'UPS_MI_BPM'
    UPS_MI_BPM_FLAT = 'UPS_MI_BPM_Flat'
    UPS_MI_BPM_PARCEL = 'UPS_MI_BPM_Parcel'
    UPS_MI_FIRST_CLASS = 'UPS_MI_First_Class'
    UPS_MI_FLAT = 'UPS_MI_Flat'
    UPS_MI_IRREGULAR = 'UPS_MI_Irregular'
    UPS_MI_MACHINABLE = 'UPS_MI_Machinable'
    UPS_MI_MEDIA_MAIL = 'UPS_MI_MEDIA_MAIL'
    UPS_MI_PARCEL_POST = 'UPS_MI_Parcel_Post'
    UPS_MI_PRIORITY = 'UPS_MI_Priority'
    UPS_MI_STANDARD_FLAT = 'UPS_MI_Standard_Flat'
    UPS_PAD_PAK = 'UPS_Pad_Pak'
    UPS_PALLET = 'UPS_Pallet'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CarrierAccountServiceLevel:
    r"""Contains details regarding the service level for the carrier account."""
    name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name'), 'exclude': lambda f: f is None }})
    r"""Service level name, e.g. `Priority Mail` or `FedEx Ground®`.
    A service level commonly defines the transit time of a Shipment (e.g., Express vs. Standard), along with other properties. 
    These names vary depending on the provider.<br>
    See <a href=\"#tag/Service-Levels\">Service Levels</a>.
    """
    token: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('token'), 'exclude': lambda f: f is None }})
    r"""Service level token, e.g. `usps_priority` or `fedex_ground`.<br>
    See <a href=\"#tag/Service-Levels\">Service Levels</a>.
    """
    supports_return_labels: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('supports_return_labels'), 'exclude': lambda f: f is None }})
    r"""Whether or not the service level supports return labels."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelPurolatorEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | purolator_ground | Ground|
    | purolator_ground9_am | Ground 9am|
    | purolator_ground1030_am | Ground 10:30am|
    | purolator_ground_distribution | Ground Distribution|
    | purolator_ground_evening | Ground Evening|
    | purolator_ground_us | Ground US|
    | purolator_express | Express|
    | purolator_express9_am | Express 9am|
    | purolator_express1030_am | Express 10am|
    | purolator_express_evening | Express Evening|
    | purolator_express_us | Express US|
    | purolator_express_us9_am | Express US 9am|
    | purolator_express_us1030_am | Express US 10:30am|
    | purolator_express_us1200 | Express US 12pm|
    | purolator_express_international | Express International|
    | purolator_express_international9_am | Express International 9am|
    | purolator_express_international1030_am | Express International 10:30am|
    | purolator_express_international1200 | Express International 12pm|
    """
    PUROLATOR_GROUND = 'purolator_ground'
    PUROLATOR_GROUND9_AM = 'purolator_ground9_am'
    PUROLATOR_GROUND1030_AM = 'purolator_ground1030_am'
    PUROLATOR_GROUND_DISTRIBUTION = 'purolator_ground_distribution'
    PUROLATOR_GROUND_EVENING = 'purolator_ground_evening'
    PUROLATOR_GROUND_US = 'purolator_ground_us'
    PUROLATOR_EXPRESS = 'purolator_express'
    PUROLATOR_EXPRESS9_AM = 'purolator_express9_am'
    PUROLATOR_EXPRESS1030_AM = 'purolator_express1030_am'
    PUROLATOR_EXPRESS_EVENING = 'purolator_express_evening'
    PUROLATOR_EXPRESS_US = 'purolator_express_us'
    PUROLATOR_EXPRESS_US9_AM = 'purolator_express_us9_am'
    PUROLATOR_EXPRESS_US1030_AM = 'purolator_express_us1030_am'
    PUROLATOR_EXPRESS_US1200 = 'purolator_express_us1200'
    PUROLATOR_EXPRESS_INTERNATIONAL = 'purolator_express_international'
    PUROLATOR_EXPRESS_INTERNATIONAL9_AM = 'purolator_express_international9_am'
    PUROLATOR_EXPRESS_INTERNATIONAL1030_AM = 'purolator_express_international1030_am'
    PUROLATOR_EXPRESS_INTERNATIONAL1200 = 'purolator_express_international1200'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelePostGlobalEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | rr_donnelley_domestic_economy_parcel | Domestic Economy Parcel|
    | rr_donnelley_domestic_priority_parcel | Domestic Priority Parcel |
    | rr_donnelley_domestic_parcel_bpm | Domestic Parcel BPM|
    | rr_donnelley_priority_domestic_priority_parcel_bpm | Domestic Priority Parcel BPM|
    | rr_donnelley_priority_parcel_delcon | International Priority Parcel DelCon|
    | rr_donnelley_priority_parcel_nondelcon | International Priority Parcel NonDelcon|
    | rr_donnelley_economy_parcel | Economy Parcel Service |
    | rr_donnelley_ipa | International Priority Airmail (IPA)|
    | rr_donnelley_courier | International Courier|
    | rr_donnelley_isal | International Surface Air Lift (ISAL)|
    | rr_donnelley_epacket | e-Packet|
    | rr_donnelley_pmi | Priority Mail International|
    | rr_donnelley_emi | Express Mail International|
    """
    RR_DONNELLEY_DOMESTIC_ECONOMY_PARCEL = 'rr_donnelley_domestic_economy_parcel'
    RR_DONNELLEY_DOMESTIC_PRIORITY_PARCEL = 'rr_donnelley_domestic_priority_parcel'
    RR_DONNELLEY_DOMESTIC_PARCEL_BPM = 'rr_donnelley_domestic_parcel_bpm'
    RR_DONNELLEY_PRIORITY_DOMESTIC_PRIORITY_PARCEL_BPM = 'rr_donnelley_priority_domestic_priority_parcel_bpm'
    RR_DONNELLEY_PRIORITY_PARCEL_DELCON = 'rr_donnelley_priority_parcel_delcon'
    RR_DONNELLEY_PRIORITY_PARCEL_NONDELCON = 'rr_donnelley_priority_parcel_nondelcon'
    RR_DONNELLEY_ECONOMY_PARCEL = 'rr_donnelley_economy_parcel'
    RR_DONNELLEY_IPA = 'rr_donnelley_ipa'
    RR_DONNELLEY_COURIER = 'rr_donnelley_courier'
    RR_DONNELLEY_ISAL = 'rr_donnelley_isal'
    RR_DONNELLEY_EPACKET = 'rr_donnelley_epacket'
    RR_DONNELLEY_PMI = 'rr_donnelley_pmi'
    RR_DONNELLEY_EMI = 'rr_donnelley_emi'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from shippo import utils
from typing import Any, List, Optional


class Status(str, Enum):
    r"""`INVALID` batch shipments cannot be purchased and will have to be removed, fixed, and added to the batch again.<br>
    `VALID` batch shipments can be purchased. <br>
    Batch shipments with the status `TRANSACTION_FAILED` were not able to be purchased and the error will be displayed on the message field<br> 
    `INCOMPLETE` batch shipments have an issue with the Address and will need to be removed, fixed, and added to the batch again.
    """
    INVALID = 'INVALID'
    VALID = 'VALID'
    INCOMPLETE = 'INCOMPLETE'
    TRANSACTION_FAILED = 'TRANSACTION_FAILED'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class BatchShipment:
    object_id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_id') }})
    r"""Object ID of this batch shipment. Can be used in the remove_shipments endpoint."""
    shipment: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('shipment') }})
    r"""Object ID of the shipment object created for this batch shipment."""
    status: Status = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status') }})
    r"""`INVALID` batch shipments cannot be purchased and will have to be removed, fixed, and added to the batch again.<br>
    `VALID` batch shipments can be purchased. <br>
    Batch shipments with the status `TRANSACTION_FAILED` were not able to be purchased and the error will be displayed on the message field<br> 
    `INCOMPLETE` batch shipments have an issue with the Address and will need to be removed, fixed, and added to the batch again.
    """
    carrier_account: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('carrier_account'), 'exclude': lambda f: f is None }})
    r"""Object ID of the carrier account to be used for this shipment (will override batch default)"""
    metadata: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    r"""A string of up to 100 characters that can be filled with any additional information you want
    to attach to the object.
    """
    servicelevel_token: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('servicelevel_token'), 'exclude': lambda f: f is None }})
    r"""A token that sets the shipping method for the batch, overriding the batch default.
    Servicelevel tokens can be found <a href=\"#tag/Service-Levels\">in this list</a> 
    or <a href=\"#operation/ListCarrierAccounts\">at this endpoint</a>.
    """
    messages: Optional[List[Any]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('messages'), 'exclude': lambda f: f is None }})
    r"""List of Shipment and Transaction error messages."""
    transaction: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('transaction'), 'exclude': lambda f: f is None }})
    r"""Object ID of the transaction object created for this batch shipment."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class UPSReferenceFields:
    prefix: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('prefix'), 'exclude': lambda f: f is None }})
    r"""Custom prefix text."""
    value: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('value'), 'exclude': lambda f: f is None }})
    r"""Label reference text. 35 character limit."""
    ref_sort: Optional[int] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('ref_sort'), 'exclude': lambda f: f is None }})
    r"""Order UPS reference fields are printed on ZPL labels. For UPS shipments, if you choose to set `ref_sort` for one reference, you must set `ref_sort` for all other supported UPS references using unique integers."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class TrackingStatusEnum(str, Enum):
    r"""Indicates the high level status of the shipment."""
    UNKNOWN = 'UNKNOWN'
    PRE_TRANSIT = 'PRE_TRANSIT'
    TRANSIT = 'TRANSIT'
    DELIVERED = 'DELIVERED'
    RETURNED = 'RETURNED'
    FAILURE = 'FAILURE'



from typing import Union

import requests

from .types import Hooks, BeforeRequestHook, BeforeRequestContext


# This file is only ever generated once on the first generation and then is free to be modified.
# Any hooks you wish to add should be registered in the init_hooks function. Feel free to define them
# in this file or in separate files in the hooks folder.


def add_shippo_token_to_auth_header_if_missing(request: requests.PreparedRequest):
    auth_header = request.headers.get("authorization")
    if auth_header is not None and auth_header.startswith("shippo_"):
        request.headers["authorization"] = f"ShippoToken {auth_header}"


class ShippoAuthBeforeRequestHook(BeforeRequestHook):

    def before_request(
            self, hook_ctx: BeforeRequestContext, request: requests.PreparedRequest
    ) -> Union[requests.PreparedRequest, Exception]:
        add_shippo_token_to_auth_header_if_missing(request)
        return request


def init_hooks(hooks: Hooks):
    # pylint: disable=unused-argument
    """Add hooks by calling hooks.register{sdk_init/before_request/after_success/after_error}Hook 
    with an instance of a hook that implements that specific Hook interface
    Hooks are registered per SDK instance, and are valid for the lifetime of the SDK instance"""
    hooks.register_before_request_hook(ShippoAuthBeforeRequestHook())



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelCorreosEspanaEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | correos_standard_home| Paquete Estándar|
    | correos_premium_home | Paquete Premium|
    """
    CORREOS_STANDARD_HOME = 'correos_standard_home'
    CORREOS_PREMIUM_HOME = 'correos_premium_home'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelDPDUKEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | dpd_uk_ship_to_shop | Ship to shop|
    | dpd_uk_1030 | Door to door 10.30 next day|
    | dpd_uk_1200 | Door to door 12.00 next day|
    | dpd_uk_saturday | Saturday Delivery|
    | dpd_uk_saturday_1030 | Saturday Delivery 10.30|
    | dpd_uk_saturday_1200 | Saturday Delivery 12.00|
    | dpd_uk_sunday | Sunday Delivery|
    | dpd_uk_sunday_1030 | Sunday Delivery 10.30|
    | dpd_uk_sunday_1200 | Sunday Delivery 12.00|
    | dpd_uk_next_day | Next day|
    | dpd_uk_two_day | Two day|
    | dpd_uk_classic | DPD Classic|
    | dpd_uk_air_classic | DPD Air Classic|
    | dpd_uk_air_express | DPD Air Express|
    | dpd_uk_direct | DPD Direct|
    | dpd_uk_direct_tracked_mail| DPD Tracked Mail|
    | dpd_uk_pickup_returns | Pickup Returns|
    """
    DPD_UK_SHIP_TO_SHOP = 'dpd_uk_ship_to_shop'
    DPD_UK_1030 = 'dpd_uk_1030'
    DPD_UK_1200 = 'dpd_uk_1200'
    DPD_UK_SATURDAY = 'dpd_uk_saturday'
    DPD_UK_SATURDAY_1030 = 'dpd_uk_saturday_1030'
    DPD_UK_SATURDAY_1200 = 'dpd_uk_saturday_1200'
    DPD_UK_SUNDAY = 'dpd_uk_sunday'
    DPD_UK_SUNDAY_1030 = 'dpd_uk_sunday_1030'
    DPD_UK_SUNDAY_1200 = 'dpd_uk_sunday_1200'
    DPD_UK_NEXT_DAY = 'dpd_uk_next_day'
    DPD_UK_TWO_DAY = 'dpd_uk_two_day'
    DPD_UK_CLASSIC = 'dpd_uk_classic'
    DPD_UK_AIR_CLASSIC = 'dpd_uk_air_classic'
    DPD_UK_AIR_EXPRESS = 'dpd_uk_air_express'
    DPD_UK_DIRECT = 'dpd_uk_direct'
    DPD_UK_DIRECT_TRACKED_MAIL = 'dpd_uk_direct_tracked_mail'
    DPD_UK_PICKUP_RETURNS = 'dpd_uk_pickup_returns'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import requests as requests_http
from .sdkconfiguration import SDKConfiguration
from shippo import utils
from shippo._hooks import AfterErrorContext, AfterSuccessContext, BeforeRequestContext, HookContext
from shippo.models import components, errors, operations
from typing import Optional

class Webhooks:
    r"""Webhooks are a way for Shippo to notify your application when a specific event occurs. For example, when a label is purchased or when a shipment tracking status has changed. You can use webhooks to trigger actions in your application, such as sending an email or updating a database.
    <SchemaDefinition schemaRef=\"#/components/schemas/Webhook\"/>

    # Webhook Payload
    The payload is the body of the POST request Shippo sends to the URL specified at the time of webhook registration.
    <SchemaDefinition schemaRef=\"#/components/schemas/WebhookPayload\"/>
    """
    sdk_configuration: SDKConfiguration

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        self.sdk_configuration = sdk_config
        
    
    
    def create_webhook(self, request: components.WebhookUpdateRequest) -> components.Webhook:
        r"""Create a new webhook
        Creates a new webhook to send notifications to a URL when a specific event occurs.
        """
        hook_ctx = HookContext(operation_id='createWebhook', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/webhooks'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        req_content_type, data, form = utils.serialize_request_body(request, components.WebhookUpdateRequest, "request", False, False, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        if data is None and form is None:
            raise Exception('request body is required')
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('POST', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 201:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.Webhook])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def list_webhooks(self) -> components.WebhookPaginatedList:
        r"""List all webhooks
        Returns a list of all webhooks you have created.
        """
        hook_ctx = HookContext(operation_id='listWebhooks', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/webhooks'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('GET', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.WebhookPaginatedList])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def get_webhook(self, webhook_id: str) -> components.Webhook:
        r"""Retrieve a specific webhook
        Returns the details of a specific webhook using the webhook object ID.
        """
        hook_ctx = HookContext(operation_id='getWebhook', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.GetWebhookRequest(
            webhook_id=webhook_id,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(base_url, '/webhooks/{webhookId}', request)
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('GET', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.Webhook])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def update_webhook(self, webhook_id: str, webhook_update_request: components.WebhookUpdateRequest) -> components.Webhook:
        r"""Update an existing webhook
        Updates an existing webhook using the webhook object ID.
        """
        hook_ctx = HookContext(operation_id='updateWebhook', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.UpdateWebhookRequest(
            webhook_id=webhook_id,
            webhook_update_request=webhook_update_request,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(base_url, '/webhooks/{webhookId}', request)
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        req_content_type, data, form = utils.serialize_request_body(request, operations.UpdateWebhookRequest, "webhook_update_request", False, False, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        if data is None and form is None:
            raise Exception('request body is required')
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('PUT', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.Webhook])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def delete_webhook(self, webhook_id: str):
        r"""Delete a specific webhook
        Deletes a specific webhook using the webhook object ID.
        """
        hook_ctx = HookContext(operation_id='deleteWebhook', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.DeleteWebhookRequest(
            webhook_id=webhook_id,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(base_url, '/webhooks/{webhookId}', request)
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers['Accept'] = '*/*'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('DELETE', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 204:
            pass
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelEvriUKEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | hermes_uk_courier_service | Courier Collection|
    | hermes_uk_parcelshop_dropoff | ParcelShop Drop-Off|
    | hermes_uk_parcelshop_dropoff_nextday | ParcelShop Drop-Off Next Day|
    | hermes_uk_postable | Postable|
    | hermes_uk_postable_nextday | Postable Next Day|
    """
    HERMES_UK_COURIER_SERVICE = 'hermes_uk_courier_service'
    HERMES_UK_PARCELSHOP_DROPOFF = 'hermes_uk_parcelshop_dropoff'
    HERMES_UK_PARCELSHOP_DROPOFF_NEXTDAY = 'hermes_uk_parcelshop_dropoff_nextday'
    HERMES_UK_POSTABLE = 'hermes_uk_postable'
    HERMES_UK_POSTABLE_NEXTDAY = 'hermes_uk_postable_nextday'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CoreRate:
    amount: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('amount'), 'exclude': lambda f: f is None }})
    r"""Final Rate price, expressed in the currency used in the sender's country."""
    amount_local: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('amount_local'), 'exclude': lambda f: f is None }})
    r"""Final Rate price, expressed in the currency used in the recipient's country."""
    currency: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('currency'), 'exclude': lambda f: f is None }})
    r"""Currency used in the sender's country, refers to `amount`.
    The <a href=\"http://www.xe.com/iso4217.php\">official ISO 4217</a> currency codes are used, e.g. `USD` or `EUR`.
    """
    currency_local: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('currency_local'), 'exclude': lambda f: f is None }})
    r"""Currency used in the recipient's country, refers to `amount_local`.
    The <a href=\"http://www.xe.com/iso4217.php\">official ISO 4217</a> currency codes are used, e.g. `USD` or \"EUR\".
    """
    object_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_id'), 'exclude': lambda f: f is None }})
    r"""Unique identifier of the Rate object."""
    provider: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('provider'), 'exclude': lambda f: f is None }})
    r"""Carrier offering the rate, e.g., `FedEx` or `Deutsche Post DHL`."""
    carrier_account: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('carrier_account'), 'exclude': lambda f: f is None }})
    r"""Object ID of the carrier account that has been used to retrieve the rate."""
    servicelevel_name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('servicelevel_name'), 'exclude': lambda f: f is None }})
    r"""Service level name, e.g. `Priority Mail` or `FedEx Ground®`.
    A service level commonly defines the transit time of a Shipment (e.g., Express vs. Standard), 
    along with other properties.  These names vary depending on the provider.<br>
    See <a href=\"#tag/Service-Levels\">Service Levels</a>.
    """
    servicelevel_token: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('servicelevel_token'), 'exclude': lambda f: f is None }})
    r"""Token of the Rate's servicelevel, e.g. `usps_priority` or `fedex_ground`.
    See <a href=\"#tag/Service-Levels\">servicelevels</a>.
    """
    

